SystemOrganization addCategory: #'Math-Core-Distribution'!

Object subclass: #DhbCongruentialRandomNumberGenerator
	instanceVariableNames: 'constant modulus multiplicator seed'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:30'!
constant: aNumber1 multiplicator: aNumber2 modulus: aNumber3 

	^super new 
		initialize: aNumber1
		multiplicator: aNumber2
		modulus: aNumber3! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:30'!
new
	"Create a new instance of the receiver with D. Knuth's constants."
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new initialize.
					 UniqueInstance setSeed: 1.
				   ].
	^UniqueInstance! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:31'!
seed: aNumber
	"Create a new instance of the receiver with given seed using D. Knuth's constants."
	^ super new initialize; setSeed: aNumber; yourself! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 19:30'!
floatValue
	"Answer the next pseudo-random value between 0 and 1."
	^self value asFloat / modulus! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'StephaneDucasse 3/2/2016 19:30'!
initialize
	
	self initialize: 2718281829.0 multiplicator: 3141592653.0 modulus: 4294967296.0.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'StephaneDucasse 3/2/2016 19:30'!
initialize: aNumber1 multiplicator: aNumber2 modulus: aNumber3
	
	constant := aNumber1.
	modulus := aNumber2.
	multiplicator := aNumber3.
	self setSeed: 1.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 19:30'!
integerValue: anInteger
	"Answer a random integer between 0 and the anInteger."
	^( self value  \\ ( anInteger * 1000)) // 1000! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'transformation' stamp: 'StephaneDucasse 3/2/2016 19:29'!
setSeed: aNumber
	"Set the seed of the receiver to aNumber."
	seed := aNumber.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 19:29'!
value
	"Answer the next pseudo-random value."
	seed := ( seed * multiplicator + constant) \\ modulus.
	^seed! !

Object subclass: #DhbErfApproximation
	instanceVariableNames: 'constant series norm'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

!DhbErfApproximation commentStamp: 'StephaneDucasse 3/2/2016 09:04' prior: 0!
A DhbErfApproximation is a five term approximation to erf(x). This is a singleton class encapsulating the approximation. 

Use with 

DhbErfApproximation new value: aNumber

to receive erf(x). The result of the error function lies between zero and one.

An extension to Number exists, so the simpler use is

aNumber errorFunction 

which produces the same result.

If you need the error function as a function, you will need to enclose it in a block as:

| errorFunction |
errorFunction := [:x | x errorFunction].

Instance variables constant and series are part of the approximation formula. norm is a scale factor to make erf(infinity) = 1.

The error function is the Cumulative Distribution of the standard normal distribution. Thus, erf(x) represents the probability of a random variable with standard normal distribution being less than x. The approximation used is credited to Abramowitz and Stegun's Handbook of Mathematical Functions. The error function is detailed in Chapter 7. The implementation is detailed in Besset's book Section 2.3

Additional resources available from NIST Digital Library of Mathematics at:
http://dlmf.nist.gov/7


(c) Copyrights Didier BESSET, 1999.
!

!DhbErfApproximation class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:04'!
new
	"Answer a unique instance. Create it if it does not exist."
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize ].
	^UniqueInstance! !

!DhbErfApproximation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"ensure a new UniqueInstance is built when used next"
	UniqueInstance := nil.! !

!DhbErfApproximation methodsFor: 'initialization' stamp: 'StephaneDucasse 3/2/2016 09:04'!
initialize
	constant := 0.2316419.
	norm := 1 / (Float pi * 2) sqrt.
	series := DhbPolynomial
		coefficients:
			#(0.31938153 -0.356563782 1.781477937 -1.821255978 1.330274429)! !

!DhbErfApproximation methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:04'!
normal: aNumber
	"Computes the value of the Normal distribution for aNumber"

	^ [ (aNumber squared * -0.5) exp * norm ]
		on: Error
		do: [ :signal | signal return: 0 ]! !

!DhbErfApproximation methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:05'!
value: aNumber
	"Answer erf(aNumber) using an approximation from Abramovitz and Stegun, Handbook of Mathematical Functions."

	| t |
	aNumber = 0
		ifTrue: [ ^ 0.5 ].
	aNumber > 0
		ifTrue: [ ^ 1
				- (self value: aNumber negated) ].
	aNumber < -20
		ifTrue: [ ^ 0 ].
	t := 1 / (1 - (constant * aNumber)).
	^ (series value: t) * t
		* (self normal: aNumber)! !

Object subclass: #DhbMitchellMooreGenerator
	instanceVariableNames: 'randoms lowIndex highIndex'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:29'!
constants: anArray lowIndex: anInteger

	^super new initialize: anArray lowIndex: anInteger! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:29'!
default

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator new.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
generateSeeds: congruentialGenerator
	"Private- "
	^self constants: ((1 to: 55) collect: [:n | congruentialGenerator floatValue])
		lowIndex: 24! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := self default].
	^UniqueInstance! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset: anInteger
	"Reset the unique instance used for the default series"
	UniqueInstance := self seed: anInteger.! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 19:29'!
seed: anInteger

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator seed: anInteger.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 19:28'!
floatValue

	| x |
	x := (randoms at: lowIndex) + (randoms at: highIndex).
	x < 1.0 ifFalse: [x := x - 1.0].
	randoms at: highIndex put: x.
	highIndex := highIndex + 1.
	highIndex > randoms size ifTrue: [highIndex := 1].
	lowIndex := lowIndex + 1.
	lowIndex > randoms size ifTrue: [lowIndex := 1].
	^x! !

!DhbMitchellMooreGenerator methodsFor: 'initialization' stamp: 'StephaneDucasse 3/2/2016 19:28'!
initialize: anArray lowIndex: anInteger
	
	randoms := anArray.
	lowIndex := anInteger.
	highIndex := randoms size.
	^self! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 19:28'!
integerValue: anInteger

	^( self floatValue * anInteger) truncated! !

Object subclass: #DhbProbabilityDensity
	instanceVariableNames: 'flatGenerator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

DhbProbabilityDensity subclass: #DhbNormalDistribution
	instanceVariableNames: 'mu sigma'
	classVariableNames: 'NextRandom'
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

!DhbNormalDistribution class methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 22:04'!
distributionName
	
	^'Normal distribution'! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 09:06'!
fromHistogram: aHistogram
	"Create an instance of the receiver with parameters estimated from the given histogram using best guesses. This method can be used to find the initial values for a fit."

	^ self
		new: aHistogram average
		sigma: aHistogram standardDeviation! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:04'!
new
	"Create a new instance of the receiver with mu=0 and sigma=1."
	^self new: 0 sigma: 1! !

!DhbNormalDistribution class methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 22:04'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma."
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbNormalDistribution class methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 22:05'!
random
	"Answer a random number distributed according to a (0,1) normal distribution."
	| v1 v2 w y |
	NextRandom isNil
		ifTrue: [ [ v1 := Number random * 2 - 1.
					v2 := Number random * 2 - 1.
					w := v1 squared + v2 squared.
					w > 1 ] whileTrue: [].
				  y := ( ( w ln * 2 negated) / w) sqrt.
				v1 := y * v1.
				NextRandom := y * v2.
				]
		ifFalse:[ v1 :=NextRandom.
				  NextRandom := nil.
				].
	^v1! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:38'!
average
	"Answer the average of the receiver."
	^mu! !

!DhbNormalDistribution methodsFor: 'transformation' stamp: 'StephaneDucasse 3/1/2016 21:38'!
changeParametersBy: aVector
	"Modify the parameters of the receiver by aVector."
	mu := mu + ( aVector at: 1).
	sigma := sigma + ( aVector at: 2).! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
distributionValue: aNumber
	"Answers the probability of observing a random variable distributed according to the receiver with a value lower than or equal to aNumber."
	^DhbErfApproximation new value: ( ( aNumber - mu) / sigma)! !

!DhbNormalDistribution methodsFor: 'initialization' stamp: 'StephaneDucasse 3/2/2016 09:07'!
initialize: aNumber1 sigma: aNumber2

	mu := aNumber1.
	sigma := aNumber2.
	^self! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
kurtosis
	"Answer the kurtosis of the receiver."
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
parameters

	^Array with: mu with: sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
random
	"Answer a random number distributed accroding to the receiver."
	^self class random * sigma + mu! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
skewness
	"Answer the skewness of the receiver."
	^0! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:39'!
standardDeviation
	"Answer the standard deviation of the receiver."
	^sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:07'!
value: aNumber
	"Answers the probability that a random variable distributed according to the receiver gives a value between aNumber and aNumber + espilon (infinitesimal interval)."
	
	^( DhbErfApproximation new normal: (aNumber - mu) / sigma) / sigma! !

!DhbNormalDistribution methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:40'!
valueAndGradient: aNumber
	"Answers an Array containing the value of the receiver at aNumber and the gradient of the receiver's respective to the receiver's parameters evaluated at aNumber."
	| dp y |
	y := ( aNumber - mu) / sigma.
	dp := ( DhbErfApproximation new normal: y) / sigma.
	^Array with: dp
		   with: ( DhbVector with: dp * y / sigma
							 with: dp * ( y squared - 1) / sigma)! !

!DhbProbabilityDensity class methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 22:04'!
distributionName
	
	^'Unknown distribution'! !

!DhbProbabilityDensity class methodsFor: 'creation' stamp: 'StephaneDucasse 3/2/2016 09:05'!
fromHistogram: aHistogram
	"Create an instance of the receiver with parameters estimated from the given histogram using best guesses. This method can be used to find the initial values for a fit. Default returns nil (must be implemented by subclass)."
	self flag: #todo. "Why not subclassResponsibility?"
	^ nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:08'!
acceptanceBetween: aNumber1 and: aNumber2
	"Answers the probability of observing a random variable distributed according to the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2. "
	
	^ (self distributionValue: aNumber2) - (self distributionValue: aNumber1)! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:38'!
approximatedValueAndGradient: aNumber
	"Private - gradients an Array containing the value of the receiver at aNumber and the gradient of the receiver's respective to the receiver's parameters evaluated at aNumber.
	 The gradient is computed by approximation."
	| delta parameters dp gradient n |
	parameters := self parameters.
	n := parameters size.
	dp := self value: aNumber.
	delta := Array new: n.
	delta atAllPut: 0.
	gradient := DhbVector new: n.
	1 to: n do:
		[ :k |
		  delta at: k put: ( parameters at: k) * 0.0001.
		  self changeParametersBy: delta.
		  gradient at: k put: ( ( ( self value: aNumber) - dp) / ( delta at: k)).
		  delta at: k put: ( delta at: k ) negated.
		  k > 1
			ifTrue: [ delta at: ( k - 1) put: 0].
		].
	self changeParametersBy: delta.
	^Array with: dp with: gradient! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:35'!
average
	"Answer the average of the receiver."
	self subclassResponsibility.! !

!DhbProbabilityDensity methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector 
	self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 21:35'!
distributionFunction

	^DhbProbabilityDistributionFunction density: self! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:37'!
distributionValue: aNumber
	"Answers the probability of observing a random variable distributed according to the receiver with a value lower than or equal to aNumber."
	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'accessing' stamp: 'StephaneDucasse 3/2/2016 13:10'!
flatGenerator: aGenerator
	"To change the default generator (which by default is flat between 0 and 1).
	aGenerator should know how to answer floatValue."
	
	flatGenerator := aGenerator! !

!DhbProbabilityDensity methodsFor: 'initialize' stamp: 'StephaneDucasse 3/2/2016 13:07'!
initialize

	super initialize.
	flatGenerator := DhbMitchellMooreGenerator new.! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:09'!
inverseDistributionValue: aNumber
	"Answer the number whose distribution value is aNumber.
	 NOTE: Subclass MUST NOT overwrite this method."
	^(aNumber between: 0 and: 1)
			ifTrue: [ self privateInverseDistributionValue: aNumber]
			ifFalse:[ self error: 'Illegal argument for inverse distribution value']! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:36'!
kurtosis
	"Answer the kurtosis of the receiver.
	Undefined. Must be implemented by subclass."
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:36'!
parameters
	"Returns an Array containing the parameters of the distribution. 
	It is used to print out the distribution and for fitting."

	^self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'display' stamp: 'StephaneDucasse 3/1/2016 21:35'!
printOn: aStream 

	| params |
	aStream nextPutAll: self class distributionName.
	(params := self parameters) notNil
		ifTrue: 
			[| first |
			first := true.
			aStream nextPut: $(.
			params
				do: 
					[:each | 
					first
						ifTrue: [first := false]
						ifFalse: [aStream nextPut: $,].
					aStream space.
					each printOn: aStream].
			aStream nextPut: $)]! !

!DhbProbabilityDensity methodsFor: 'private' stamp: 'StephaneDucasse 3/1/2016 21:36'!
privateInverseDistributionValue: aNumber
	"Private - Answer the number whose distribution is aNumber.
	NOTE: Subclass may overwrite this method for faster computation."
	^( CdfNewtonZeroFinder function: [ :x | ( self distributionValue: x) - aNumber] derivative: self)
		initialValue: self average ; evaluate! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 13:10'!
random
	"Answer a random number distributed according to the receiver."
	^self privateInverseDistributionValue: flatGenerator floatValue! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:36'!
skewness
	"Answer the skewness of the receiver.
	 Undefined. Must be implemented by subclass."
	^nil! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:37'!
standardDeviation
	"Answer the standard deviation of the receiver.
	NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass."
	^self variance sqrt! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/2/2016 09:08'!
value: aNumber
	"Answers the probability that a random variable distributed according to the receiver
	gives a value between aNumber and aNumber + espilon (infinitesimal interval)."
	self subclassResponsibility! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:37'!
valueAndGradient: aNumber
	"Answers an Array containing the value of the receiver at aNumber
	 and the gradient of the receiver's respective to the receiver's
	 parameters evaluated at aNumber."
	^self approximatedValueAndGradient: aNumber! !

!DhbProbabilityDensity methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:37'!
variance
	"Answer the variance of the receiver.
	NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass."
	^self standardDeviation squared! !

PackageManifest subclass: #ManifestMathCoreDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Core-Distribution'!

!ManifestMathCoreDistribution commentStamp: '<historical>' prior: 0!
I store metadata for this package. These meta data are used by other tools such as the SmalllintManifestChecker and the critics Browser!

!ManifestMathCoreDistribution class methodsFor: 'code-critics' stamp: 'StephaneDucasse 3/2/2016 09:06'!
ruleRBCodeCruftLeftInMethodsRuleV1FalsePositive
	^ #(#(#(#RGMethodDefinition #(#'DhbProbabilityDensity class' #fromHistogram: #true)) #'2016-03-02T09:06:24.499812+01:00') )! !
